#pragma once
#include "robot/i_robot_io.h"
#include <kr2_io_api/api_v1/io_interface.h>
#include <kr2_program_api/api_v1/program_interface.h>

namespace spin
{
    class RobotIO : public IRobotIO
    {
    public:
        RobotIO(boost::shared_ptr<kr2_program_api::ProgramInterface> api) : api_{api} {}

        void writeDigitalOutput(const int pin, const bool value) const override
        {
            auto ioBoard_DO = kr2rc_api::IOData::SysId::OPORT(pin - 1, kr2rc_api::IOData::SysId::IOBOARD, kr2rc_api::IOData::SysId::DIGITAL);
            bool ioBoard_DO_value = value;
            unsigned int ioBoard_DO_config_id = 0;

            const std::array<kr2rc_api::IOData::GPIOInt64, 1> digital_io{{{ioBoard_DO, ioBoard_DO_value, ioBoard_DO_config_id}}};
            kr2rc_api::IOData::CmdTXGPIOParams params;

            auto result = api_->rc_api_->iob_data_->cmd_TX_GPIO(params, nullptr, 0, digital_io.data(), digital_io.size(), nullptr, 0);
        }

        std::optional<bool> readDigitalOutput(const int pin) const override
        {
            const auto ioBoard_DO = kr2rc_api::IOData::SysId::OPORT(pin - 1, kr2rc_api::IOData::SysId::IOBOARD, kr2rc_api::IOData::SysId::DIGITAL);
            const int io_count = api_->rc_api_->iob_data_->read_N_GPIOInt64();

            for (int i = 0; i < io_count; ++i)
            {
                auto info = api_->rc_api_->iob_data_->read_GPIOInt64(i);
                if (info->gpio_id_ == ioBoard_DO)
                {
                    return static_cast<bool>(info->value_);
                }
            }
            return std::nullopt;
        }

    private:
        boost::shared_ptr<kr2_program_api::ProgramInterface> api_;
    };
} // namespace spin
